#!/bin/sh
# Installation script for redhat

IF_UTIL_FOUND=0
IF_UTIL_TYPE=none

DEFAULT_DIR=/etc/sysconfig
DEFAULT_FILE=hogwash

echo "Making installation directories (if they currently don't exist)..."
mkdir -p /etc/hogwash
mkdir -p /etc/hogwash/rules
mkdir -p /var/log/hogwash
mkdir -p /sbin
mkdir -p /etc/rc.d/init.d
mkdir -p /etc/rc.d/rc0.d
mkdir -p /etc/rc.d/rc1.d
mkdir -p /etc/rc.d/rc2.d
mkdir -p /etc/rc.d/rc3.d
mkdir -p /etc/rc.d/rc4.d
mkdir -p /etc/rc.d/rc5.d
mkdir -p /etc/rc.d/rc6.d

echo "Installing files..."

install -m 755 hogwash /sbin/hogwash
#install -m 755 ifconfig-nostack /sbin/ifconfig-nostack
install -m 550 rules.2 /etc/hogwash/rules/hogwash-rules.v00001
install -m 550 scontrol.conf /etc/hogwash/scontrol.conf
install -m 755 hogwash.rc /etc/rc.d/init.d/hogwash
install -m 644 man/hogwash.8.gz /usr/share/man/man8
install -m 644 man/hogwash.5.gz /usr/share/man/man5

echo "Setting up startup and shutdown runlevels...."

chkconfig --add hogwash
chkconfig --list hogwash

echo "Probing for interface control utilities..."

if [ $IF_UTIL_FOUND = 0 ]; then
  echo -n "Checking for ip: "
  which ip > /dev/null 2>&1
  if [ $? = 1 ]; then
    echo "no."
    IF_UTIL_FOUND=0
    IF_UTIL_TYPE=none
  else
    echo "$(which ip)"
    IF_UTIL_FOUND=1
    IF_UTIL_TYPE=ip
  fi
fi

if [ $IF_UTIL_FOUND = 0 ]; then
  echo -n "Checking for ifconfig-nostack: "
  which ifconfig-nostack > /dev/null 2>&1
  if [ $? = 1 ]; then
    echo "no."
    IF_UTIL_FOUND=0
    IF_UTIL_TYPE=none
  else
    echo "$(which ifconfig-nostack)"
    IF_UTIL_FOUND=1
    IF_UTIL_TYPE=ifconfig-nostack
  fi
fi

if [ $IF_UTIL_FOUND = 0 ]; then
  echo -n "Checking for ifconfig: "
  which ifconfig > /dev/null 2>&1
  if [ $? = 1 ]; then
    echo "no."
    IF_UTIL_FOUND=0
    IF_UTIL_TYPE=none
  else
    echo "$(which ifconfig)"
    IF_UTIL_FOUND=1
    IF_UTIL_TYPE=ifconfig
  fi
fi

if [ $IF_UTIL_FOUND = 0 ]; then
  echo "WARNING: No interface control programs were found. If you are using"
  echo "a different interface control program, after the install has finished,"
  echo "please go edit the ${DEFAULT_DIR}/${DEFAULT_FILE} file and add it."
  echo
fi

if [ $IF_UTIL_FOUND = 1 ]; then
  if [ "$IF_UTIL_TYPE" = "ip" ]; then
    IF_UTIL=$(which ip)
    IFUP_PRE_COMMAND="link set"
    IFUP_POST_COMMAND="up"
  fi
  if [ "$IF_UTIL_TYPE" = "ifconfig-nostack" ]; then
    IF_UTIL=$(which ifconfig-nostack)
    IFUP_PRE_COMMAND=""
    IFUP_POST_COMMAND="up"
  fi
  if [ "$IF_UTIL_TYPE" = "ifconfig" ]; then
    IF_UTIL=$(which ifconfig)
    IFUP_PRE_COMMAND=""
    IFUP_POST_COMMAND="up"
  fi
  if [ "$IF_UTIL_TYPE" = "none" ]; then
    IF_UTIL=""
    IFUP_PRE_COMMAND=""
    IFUP_POST_COMMAND=""
  fi
fi

echo "Building ${DEFAULT_DIR}/${DEFAULT_FILE} file..."
	if [ -e ${DEFAULT_DIR}/${DEFAULT_FILE} ]; then
		mv ${DEFAULT_DIR}/${DEFAULT_FILE} ${DEFAULT_DIR}/${DEFAULT_FILE}.old
	fi
touch ${DEFAULT_DIR}/${DEFAULT_FILE} 
echo "# Daemon name (used in the init script only)." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "PROG=hogwash" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Internal interface device name." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "INT_IF=eth0" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# External interface device name." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "EXT_IF=eth1" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Rules file to use with hogwash." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "RULES_FILE=/etc/hogwash/rules/hogwash-rules.v00001" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Directory where hogwash can find rules files." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "RULES_DIR=/etc/hogwash/rules" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Config file for stackless administration system." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "CONTROL_FILE=/etc/hogwash/scontrol.conf" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Also used with stackless administration system." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "SID_FILE=/etc/hogwash/current.sid" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Disables IPv4 Packet Forwarding." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "DISABLE_FORWARDING=yes" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Enables span port for hogwash (supplies packet copies to other IDS progr
ams" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "ENABLE_SPANPORT=no" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Span port interface device name." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "SPAN_IF=" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Necessary parameters that go before the device name, when configuring." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "IFUP_PRECOMMAND=$IFUP_PRE_COMMAND" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Necessary parameters that go after the device name, when configuring." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "IFUP_POSTCOMMAND=$IFUP_POST_COMMAND" >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "# Command to run to configure an interface." >> ${DEFAULT_DIR}/${DEFAULT_FILE}
echo "IFUTIL=$IF_UTIL" >> ${DEFAULT_DIR}/${DEFAULT_FILE}

echo "Redhat hogwash install is done! Happy packet scrubbing!"
echo "If any errors were printed above, please make sure you "
echo "are installing as root."
echo

