
#include "aesinc.h"

/***********************************************************
*  Scramble things
***********************************************************/
int AESEncrypt(CryptoRecord* CR, char* InData, char* OutData, int DataLen){
	AESData*		ae;
	int				size;
	int				i;
	char			buff[BLOCKSIZE];

#ifdef DEBUG	
	printf("Encryping with AES\n");
#endif	
	ae=CR->PrivateData;
	
	if (DataLen%BLOCKSIZE !=0 ){
		printf("DataLen isn't a multiple of the block size\n");
		return FALSE;
	}
	
	for (i=0;i<DataLen;i=i+BLOCKSIZE){
		if (!aes_encrypt(InData+i, OutData+i, &ae->encrypt_key)){
			printf("Failed to encrypt block %i with aes\n",i);
			return FALSE;
		}	
	}
					
	return TRUE;
}

/***********************************************************
*  Un-Scramble things
***********************************************************/
int AESDecrypt(CryptoRecord* CR, char* InData, char* OutData, int DataLen){
	AESData*		ae;
	int				size;
	int				i;
	char			buff[BLOCKSIZE];

#ifdef DEBUG	
	printf("Decryping with AES\n");
#endif	
	ae=CR->PrivateData;
	
	if (DataLen%BLOCKSIZE !=0 ){
		printf("DataLen isn't a multiple of the block size\n");
		return FALSE;
	}
	
	for (i=0;i<DataLen;i=i+BLOCKSIZE){
		if (!aes_decrypt(InData+i, OutData+i, &ae->decrypt_key)){
			printf("Failed to decrypt block %i with aes\n");
			return FALSE;
		}	
	}
					
	return TRUE;
}

/***********************************************************
*  Add a new key
***********************************************************/
int	AddKeyAES(CryptoRecord* CR, int ID, char* KeyString){
	AESData*	ae;
	int			i;
	char		IntBuff[16];

	/*general setup*/
	CR->InUse=TRUE;
	CR->ID=ID;
	CR->CryptoType=CRYPTO_TYPE_AES;
	CR->PrivateData=(void*)calloc(sizeof(AESData),1);
	CR->Encrypt=(CryptoFunc)AESEncrypt;
	CR->Decrypt=(CryptoFunc)AESDecrypt;
	
	
	/*init the key*/
	ae=CR->PrivateData;
	
	/*make the key*/
	if (strlen(KeyString)!=((AES_KEY_SIZE/32)*8)){
		printf("wrong key size.  Was %i expected %i characters\n",strlen(KeyString), ((AES_KEY_SIZE/32)*8));
		free(CR->PrivateData);
		CR->InUse=FALSE;		
	}
	for (i=0;i<(AES_KEY_SIZE/32);i++){
		memcpy(IntBuff, KeyString+(i*8),8);
		IntBuff[9]=0x0;
		ae->key[i]=strtol(IntBuff, NULL, 16); 
	}
#ifdef DEBUG
	printf("set key:");
	for (i=0;i<32/4;i++){
		printf("%02x",((unsigned int*)ae->key)[i]);
	}
	printf("\n");
#endif	

	if (!aes_set_blk(BLOCKSIZE, &ae->decrypt_key)){
		printf("Failed to set decrypt block size\n");
		return FALSE;
	}
	if (!aes_set_blk(BLOCKSIZE, &ae->encrypt_key)){
		printf("Failed to set encrypt block size\n");
		return FALSE;
	}	
	if (!aes_set_key((char*)ae->key, BLOCKSIZE, aes_dec, &ae->decrypt_key)){
		printf("Failed to set decrypt key\n");
		return FALSE;
	}
	if (!aes_set_key((char*)ae->key, BLOCKSIZE, aes_enc, &ae->encrypt_key)){
		printf("Failed to set encrypt key\n");
		return FALSE;
	}

}

/***********************************************************
*  Get Rid of a key
***********************************************************/
void RemoveKeyAES(CryptoRecord* CR){
	if (CR->PrivateData) free(CR->PrivateData);
	CR->InUse=FALSE;
}





