#ifndef HOGWASH_CONTROL_COMMON_H
#define HOGWASH_CONTROL_COMMON_H

#define CONTROL_MAGIC_TOKEN	0x56565656

#define CONTROL_PACKET_PING				1
#define CONTROL_PACKET_PING_REPLY		2
#define CONTROL_PACKET_BUFF_REQUEST		3
#define CONTROL_PACKET_BUFF_REPLY		4
#define CONTROL_PACKET_STAT_REQUEST		5
#define CONTROL_PACKET_STAT_REPLY		6
#define CONTROL_PACKET_SET_BUFF			7
#define CONTROL_PACKET_SET_REPLY		8
#define CONTROL_PACKET_COMMAND			9
#define CONTROL_PACKET_COMMAND_REPLY	10


#define STOCK_BUFFER_RULES			1010
#define STOCK_BUFFER_ALERT			2021
#define STOCK_BUFFER_LOG			3054
#define STOCK_BUFFER_NEW_RULES		4032
#define STOCK_BUFFER_NEW_EXEC		5055
#define STOCK_BUFFER_UALERT			6006
#define STOCK_BUFFER_ULOG			7007
#define STOCK_BUFFER_SID			8888

#define MAX_BUFFER_SEGMENT			740
#define BUFFER_MAX_RETRY			3

#define BUFFER_FLAG_OK				0
#define	BUFFER_FLAG_NOT_FOUND		1
#define BUFFER_FLAG_OPEN_ERROR		2
#define BUFFER_FLAG_END_OF_BUFFER	3
#define BUFFER_FLAG_SEEK_ERROR		4
#define BUFFER_FLAG_TIMED_OUT		5
#define BUFFER_FLAG_KEY_ERROR		6
#define BUFFER_FLAG_NOT_READY		7
#define BUFFER_FLAG_NO_STATE		8
#define BUFFER_FLAG_UNKNOWN_ERR		9

#define BUFFER_TYPE_SHARED_FILE		1
#define BUFFER_TYPE_SINGLE_FILE		2
#define BUFFER_TYPE_SHARED_MEM		3
#define BUFFER_TYPE_SINGLE_MEM		4

#define COMMAND_RESTART				1
#define COMMAND_SET_BUFFER_VERSION	2

typedef struct control_packet{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
} ControlPacket;

typedef struct control_packet_ping{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	char			PingMessage[244];
} ControlPacketPing;

#define ControlPacketPingReply	ControlPacketPing

typedef struct control_packet_buffer_reqest{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	unsigned int	Offset;
	unsigned int	Len;
	int				Flags;
	char			Zero[228];
} ControlPacketBuffRequest;

typedef struct control_packet_buffer_reply{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	unsigned int	Offset;
	unsigned int	Len;
	int				Flags;
	char			Data[MAX_BUFFER_SEGMENT];
} ControlPacketBuffReply;

typedef struct control_packet_buffer_stat_request{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	char			Zero[238];
} ControlPacketStatRequest;

typedef struct control_packet_buffer_stat_reply{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	int				BuffType;
	unsigned long	CreateTime;
	unsigned long	ModifyTime;
	unsigned long	TimeLeft;
	unsigned int	Size;

	/*General Snort Stats*/
	unsigned int	TCPCount;
	unsigned int	UDPCount;
	unsigned int	ICMPCount;
	unsigned int	ARPCount;
	unsigned int	IPV6Count;
	unsigned int	IPXCount;
	unsigned int	OtherCount;
	unsigned int	AlertCount;
	unsigned int	LogCount;
	unsigned int	DropCount;
	char			Zero[174];
	
	/*added at end for backwards compat*/
	int				BufferVersion;
} ControlPacketStatReply;

typedef struct control_packet_reset{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	char			Zero[242];
} ControlPacketReset;

typedef struct control_packet_set_buffer{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	unsigned int	Offset;
	unsigned int	Len;
	int				Flags;
	char			Data[MAX_BUFFER_SEGMENT];
} ControlPacketSetBuffer;


typedef struct control_packet_set_buffer_reply{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	int				Flags;
	unsigned int	Offset;
	unsigned int	Len;
	char			Zero[230];
} ControlPacketSetBufferReply;


typedef struct control_packet_command{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	unsigned int	CommandID;
	int				Flags;
	char			Data[230];
} ControlPacketCommand;

typedef struct control_packet_command_reply{
	unsigned int	MagicToken1;
	unsigned int	MagicToken2;
	unsigned short	PacketType;
	unsigned int	PacketID;
	unsigned int	BuffID;
	unsigned int	CommandID;
	int				Flags;
	char			Data[230];
} ControlPacketCommandReply;

#endif


