
#include "twofishinc.h"

/***********************************************************
*  Scramble things
***********************************************************/
int TwoFishEncrypt(CryptoRecord* CR, char* InData, char* OutData, int DataLen){
	int				count;
	TwoFishData*	tf;

#ifdef DEBUG	
	printf("Encryping with twofish\n");
#endif	
	tf=CR->PrivateData;
	
	count=blockEncrypt(&tf->ci, &tf->ki, InData, DataLen*8, OutData);
	if (count!= (DataLen*8)){
		printf("Failed to encrypt with code %i\n",count);
		return FALSE;
	}
		
	return TRUE;
}

/***********************************************************
*  Un-Scramble things
***********************************************************/
int TwoFishDecrypt(CryptoRecord* CR, char* InData, char* OutData, int DataLen){
	int				count;
	TwoFishData*	tf;
	
#ifdef DEBUG	
	printf("Decryping with twofish\n");
#endif	
	tf=CR->PrivateData;
	
	count=blockDecrypt(&tf->ci, &tf->ki, InData, DataLen*8, OutData);
	if (count!= (DataLen*8)){
		printf("Failed to encrypt with code %i\n",count);
		return FALSE;
	}
	return TRUE;
}

/***********************************************************
*  Add a new key
***********************************************************/
int	AddKeyTwoFish(CryptoRecord* CR, int ID, char* KeyString){
	TwoFishData*	tf;
	char			IntBuff[9];
	int				i;

	/*general setup*/
	CR->InUse=TRUE;
	CR->ID=ID;
	CR->CryptoType=CRYPTO_TYPE_TWOFISH;
	CR->PrivateData=(void*)calloc(sizeof(TwoFishData),1);
	CR->Encrypt=(CryptoFunc)TwoFishEncrypt;
	CR->Decrypt=(CryptoFunc)TwoFishDecrypt;
	
	
	/*init the key*/
	tf=CR->PrivateData;
	makeKey(&tf->ki, DIR_ENCRYPT, TWOFISH_KEY_SIZE, NULL);
	cipherInit(&tf->ci, MODE_ECB, NULL);
	
	/*make the key*/
	if (strlen(KeyString)!=((TWOFISH_KEY_SIZE/32)*8)){
		printf("wrong key size.  Was %i expected %i characters\n",strlen(KeyString), ((TWOFISH_KEY_SIZE/32)*8));
		free(CR->PrivateData);
		CR->InUse=FALSE;		
	}
	for (i=0;i<(TWOFISH_KEY_SIZE/32);i++){
		memcpy(IntBuff, KeyString+(i*8),8);
		IntBuff[9]=0x0;
		tf->ki.key32[i]=strtol(IntBuff, NULL, 16); 
	}
	reKey(&tf->ki);

	
}

/***********************************************************
*  Get Rid of a key
***********************************************************/
void RemoveKeyTwoFish(CryptoRecord* CR){
	if (CR->PrivateData) free(CR->PrivateData);
	CR->InUse=FALSE;
}





