#include "defrag.h"
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define DEBUG

#define MAX_FRAGS	65536

typedef struct frag_holder{
	long				packet_time;
	unsigned int		start_offset;
	unsigned int		end_offset;
	char				raw_packet[65536+100];
	char				last_frag;
	struct frag_holder*	next;
	struct frag_holder* age_next;
} FragHolder;

FragHolder*	Frags[MAX_FRAGS];

typedef struct _IPHdr
{
    u_int8_t ip_hlen:4, ip_ver:4;
    u_int8_t ip_tos;        /* type of service */
    u_int16_t ip_len;       /* datagram length */
    u_int16_t ip_id;        /* identification  */
    u_int16_t ip_offset;       /* fragment offset */
    u_int8_t ip_ttl;        /* time to live field */
    u_int8_t ip_proto;      /* datagram protocol */
    u_int16_t ip_csum;      /* checksum */
    struct in_addr ip_src;  /* source IP */
    struct in_addr ip_dst;  /* dest IP */
}      IPHdr;


void DeleteFrags(int ip_id){
	FragHolder*	this;
	FragHolder*	last;
	
	this = Frags[ip_id];
	Frags[ip_id] = NULL;
	while (this){
		last = this;
		this = this->next;
		free(last);
	}	
}

void FragInit(int argc, char** argv){
	bzero(Frags, sizeof(FragHolder*)*MAX_FRAGS);
}

/* Stolen from snort-1.8.6
 * Function: checksum(u_int16_t *, u_int32_t, u_int16_t *, u_int32_t)
 *
 * Purpose: Huh huh, math....
 *
 * Arguments: b1   => pointer to data set one
 *            len1 => length of data set one
 *            b2   => pointer to data set two
 *            len2 => length of data set two
 *
 * Returns: csum => the calculated checksum for the given data
 */
u_int16_t defrag_checksum(u_int16_t *b1, u_int32_t len1, u_int16_t *b2, u_int32_t len2)
{
    u_int32_t sum = 0;

    if(b1 != (u_int16_t *)NULL) 
    {
        while(len1 > 1) 
        {
            sum += *((u_int16_t *)b1 ++);

            if(sum & 0x80000000)
            {
                sum = (sum & 0xffff) + (sum >> 16);
            }

            len1 -= 2;
        }
    
        /* we'll have problems if b2 exists and len1 is odd */
        if(len1)
        {
           sum += (u_int16_t) * (u_int8_t*) b1;
        }
    }

    if(b2 != (u_int16_t*)NULL) 
    {
        while(len2 > 1) 
        {
            sum += *((u_int16_t*)b2 ++);

            if(sum & 0x80000000)
            {
                sum = (sum & 0xffff) + (sum >> 16);
            }

            len2 -= 2;
        }

        if(len2)
        {
            sum += (u_int16_t) * (u_int8_t*) b2;
        }
    }

    while(sum >> 16)
        sum = (sum & 0xffff) + (sum >> 16);

    return (u_int16_t) ~sum;
}


int ReAssemble(int FragID, PacketToProcess* ptp){
	char* 			Buff;
	FragHolder		*search, *tmp;
	IPHdr*			ip;
	char*			temp;

#ifdef DEBUG
	printf("Reassembling all the pieces\n");
#endif

	search = Frags[FragID];
	Frags[FragID]=NULL;
	Buff = ptp->raw_packet;
	
	/*copy in the first frag's Ethernet header*/
	memcpy(Buff, search->raw_packet ,14);
	Buff +=14;
	
	/*copy in the first frag's IP header*/
	ip = (IPHdr*)(search->raw_packet+14);
	memcpy(Buff, ip, (ip->ip_hlen * 4));
	Buff += (ip->ip_hlen*4);
	
	/*copy in each of the fragment's data*/
	while (search){
		printf("Buff is %i\n",Buff - ptp->raw_packet);
		printf("Copying in fragment %u-%u\n",search->start_offset, search->end_offset);		
		ip = (IPHdr*)(search->raw_packet+14);
		temp = (char*)ip;
		temp += (ip->ip_hlen * 4);
		memcpy(Buff + search->start_offset, temp, (search->end_offset - search->start_offset));
		ptp->packet_len = search->end_offset + 14;
		ptp->capture_len = search->end_offset + 14;
		printf("New length is %u\n",ptp->packet_len);
		tmp = search;
		search = search->next;
		free(tmp);	
	}	
	
	/*recalculate the checksums*/
	ip = (IPHdr*)(ptp->raw_packet+14);
	ip->ip_csum = 0x00;
	ip->ip_len= ntohs(ptp->packet_len - 14);
	ip->ip_offset = htons(ntohs(ip->ip_offset) & 0x1FFF);
	ip->ip_csum = defrag_checksum((u_short*)ip, ip->ip_hlen * 4, NULL, 0);
	
	/*send it on*/	
	return TRUE;
}

int  ProcessFrag(PacketToProcess* ptp){
	IPHdr*			ip;
	unsigned char	flags;
	unsigned int	offset;
	FragHolder*		this;
	FragHolder*		search;
	
	/*TODO: make this not depend on ethernet*/
	/*decode packet*/
	if ((*(unsigned short*)(ptp->raw_packet+12)) != 0x0008 ){
#ifdef DEBUG1	
		printf("Not an IP packet %04x\n",*(unsigned short*)(ptp->raw_packet+12));
#endif
		return FALSE;		
	}
	ip = (IPHdr*)(ptp->raw_packet+14);
	
	flags  = ntohs(ip->ip_offset) / 8192;
	offset = ntohs(ip->ip_offset) & 0x1FFF;
	if ( (offset == 0) && ((flags & 0x01)==0) ){
#ifdef DEBUG1
		printf("Not a fragment\n");
#endif	
		return FALSE;
	}

	/*fill in the struct*/
	this=(FragHolder*)calloc(sizeof(FragHolder), sizeof(char));
	this->packet_time = ptp->ts.tv_sec;
	this->start_offset = offset * 8;
	this->end_offset = this->start_offset + ntohs(ip->ip_len) - (ip->ip_hlen *4);
	this->last_frag = ((flags & 0x01)==0x00);
	/*this is wrong*/
	if (this->last_frag) this->end_offset += 20;
	/*end wrong*/
	memcpy(this->raw_packet, ptp->raw_packet, ptp->packet_len);

#ifdef DEBUG
	printf("_________________\n");
	printf("Is a fragment %u\n", ip->ip_id);
	printf("offset=%u\n", (offset*8));
	printf("Frag flags=%u\n", flags);
	printf("Starting at %u\n", this->start_offset);
	printf("Ending at %u\n", this->end_offset);
#endif	

	if (this->end_offset > 65535){
#ifdef DEBUG
		printf("Ping of death\n");
#endif	
		return TRUE;
	}

	/*send the fragment off to the cache*/
	if (!Frags[ip->ip_id]){
		/*First Piece*/
		Frags[ip->ip_id]=this;
	}else{
		/*add to the list in offset order*/
		if (this->start_offset==0){
			this->next = Frags[ip->ip_id];
			Frags[ip->ip_id] = this;
		}else{
			search = Frags[ip->ip_id];
			while (search->next){
				if (this->start_offset < search->next->start_offset){
					this->next = search->next;
					search->next = this;
					break;
				}
				search = search->next;	
			}
			if (!(search->next)){
				search->next = this;
			}
		}
	}
	
	/*check to see if we have all the pieces*/
	search = Frags[ip->ip_id];
	if (search->start_offset!=0) return TRUE;
	while(search){
#ifdef DEBUG		
		printf("%u-%u\n",search->start_offset, search->end_offset);
#endif		
		if (search->last_frag){
#ifdef DEBUG
			printf("We have all the pieces\n");
#endif			
			/*erase the frags*/
			return !ReAssemble(ip->ip_id, ptp);
		}else{
			if (!search->next) return TRUE;
			if (search->next->start_offset < search->end_offset){
#ifdef DEBUG
				printf("Overlapping fragments-kill em\n");
#endif			
				DeleteFrags(ip->ip_id);
				return TRUE;
			}else if (search->next->start_offset > search->end_offset){
				return TRUE;
			}
		}
		search=search->next;
	}
	
	/*drop this fragment*/
	return TRUE;		
}
