#include "packet.h"

#ifdef __PCAP__

/********************************************
* one for each thread
********************************************/
void* MainLoop(void* v){
	struct interface_rec*	r;

#ifdef __OPENBSD__
	int						fd1, fd2;
#endif	
	
#ifdef __FREEBSD__	
	int						packet_len;
	char					Buff[65536];
	struct pcap_pkthdr		pc;
	int						fd1, fd2;
#endif	

#ifdef __OPENBSD__
	fd1=open_interface(internal_name);
	fd2=open_interface(external_name);
	MainLoopOpenBSD(fd1, fd2);
#endif
	
#ifdef __FREEBSD__
	printf("Starting FreeBSD loop\n");
	
	fd1=open_interface("fxp0");
	fd2=open_interface("rl0");
	
	while (1){
		packet_len=-1;
		packet_len=get_packet(fd1, Buff, 65535, &pc.ts);
		if (packet_len!=-1){
#ifdef DEBUG
			printf("Got packet on interface 0 with len %i\n",r->num, packet_len);
#endif		
			pc.caplen=packet_len;
			pc.len=packet_len;
			ProcessPacket((void*)0, &pc, Buff);
		}
		
		packet_len=-1;
		packet_len=get_packet(fd2, Buff, 65535, &pc.ts);
		if (packet_len!=-1){
#ifdef DEBUG
			printf("Got packet on interface 1 with len %i\n",r->num, packet_len);
#endif		
			pc.caplen=packet_len;
			pc.len=packet_len;
			ProcessPacket((void*)1, &pc, Buff);
		}

	}
#else  
	printf("Starting Loop\n");
	r=(struct interface_rec*)v;
    if (r==&internal){
    	printf("Internal Interface %i\n", pthread_self());
		pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
		Thread1PID=getpid();
	}else{
		printf("External Interface %i\n", pthread_self());
		pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
		Thread2PID=getpid();
	}	
	pcap_loop(r->pds, -1, (pcap_handler) ProcessPacket, (char*)r->num);
#endif	
	return NULL;
}



int open_interfaces(char* internalif, char* externalif){
}
void close_interfaces(){
		CloseInterfaces();
		pass_rule_free();
		DestroyLock();

}
void main_loop(){
	pthread_create(&external.thread, NULL, MainLoop, (void*)&external);
	pthread_create(&internal.thread, NULL, MainLoop, (void*)&internal);

	printf("Waiting for joins\n");
	pthread_join(external.thread, NULL);
	printf("external joined\n");
	pthread_join(internal.thread, NULL);
	printf("internal joined\n");

}
void restart(){
		printf("Threads Joined.  Restarting Now....\n");
		CloseInterfaces();
		pass_rule_free();
		remove(PID_FILE);
		printf("Calling execv\n");
		execv(argv[0], argv);
}


/*******************************************************
 * open the libpcap devices.  Mostly stolen from snort.
 *******************************************************/
int OpenPcap(struct interface_rec* r, char* interface_name){
	char errorbuf[1024];
    struct bpf_program fcode;    
	
	printf("In OpenPcap\n");
	if (!interface_name){
		r->string = pcap_lookupdev(errorbuf);
		hogwashprint(5,"found interface %s\n", r->string);
	}else{
		r->string=interface_name;
		hogwashprint(5,"Using guess %s\n", r->string);
	}
	r->pds = pcap_open_live(r->string, SNAPLEN,
         PROMISC , READ_TIMEOUT, errorbuf);
	 /* lookup mtu */
    r->mtu = GetIfrMTU(r->string);
	hogwashprint(5,"New MTU: %i\n",r->mtu);
	if (r->mtu == -1){
		printf("Couldn't get MTU assuming 1500\n");
		r->mtu=1500;
		//return FALSE;
	}
	
	if(pcap_compile(r->pds, &fcode, NULL, 1, 0) < 0)
    {
        hogwashprint(3,"ERROR: OpenPcap() FSM compilation failed: \n\t%s\n",
                     pcap_geterr(r->pds));
        hogwashprint(3,"PCAP command: %s\n", NULL);
    }
	
    /* set the pcap filter */
    if(pcap_setfilter(r->pds, &fcode) < 0)
    {
        hogwashprint(3,"ERROR: OpenPcap() setfilter: \n\t%s\n",
                   pcap_geterr(r->pds));
    }
	
    /* get data link type */
    r->datalink = pcap_datalink(r->pds);

    if(r->datalink < 0)
    {
        hogwashprint(3,"ERROR: OpenPcap() datalink grab: \n\t%s\n",
                   pcap_geterr(r->pds));
    }
	
	return TRUE;
}

/********************************************************
*  Reset and restart all the functions.  Reread rules
* and key files.
********************************************************/
void HogwashRestart(){
	printf("Restarting.......\n");
	printf("My PID   %i\n",getpid());
	printf("Main PID %i\n",MainPID);
	printf("T1 PID   %i\n",Thread1PID);
	printf("T2 PID   %i\n",Thread2PID);
	Restart=TRUE;
	printf("Stopping internal thread....\n");
	pthread_cancel(internal.thread);
	printf("Stopping external thread....\n");
	pthread_cancel(external.thread);
	printf("Done stopping threads\n");	
	
	/*send a HUP to the other thread*/
	
}


#endif
